// STAR-CCM+ macro: multiSIM.java
// Valid from version 11.06 onwards

package macro;

import java.util.*;

import star.common.*;
import star.coremodule.SimulationApplication;

/**
 *
 * @author paco.ezquerra@cd-adapco.com
 */

public class multiSIM extends StarMacro {

    @Override
    public void execute() {
        
        int source = 0;
        Collection<Simulation> sl = SimulationApplication.getSimulations();
        Simulation[] simArray = (Simulation[]) sl.toArray(new Simulation[sl.size()]);
        
        //Pick the source simulation: Here, the one which contains the '$' character
        boolean found = false;
        for (int i = 0; i < simArray.length; i++) {
            if(simArray[i].getPresentationName().contains("$")){
              source = i;
              found = true;
            } 
           
        }

        if(!found) {
            for (int i = 0; i < simArray.length; i++) {
               simArray[i].println("--- No simulation found with character '$' ");
               simArray[i].println("--- Source simulation has index 0 ");
            }
        }
        
        simArray[source].println(" --- I am the simulation source ");
        
        // Some messagging among simulations
        
        for (int i = 0; i < simArray.length; i++) {
            // Just sending messages to own output
            simArray[i].println(" --- I am simulation number " + i);
            simArray[i].println(" --- My name is " + simArray[i].getPresentationName());
            // Send greeting message to the output windows of the rest of simulations. 
            for (int j = i + 1; j < i + simArray.length; j++) {
                simArray[j % simArray.length].println(" <<< Greetings from simulation " + simArray[i].getPresentationName());
            }
            // Run one iteration or time step
            simArray[i].getSimulationIterator().run(1);

        }
        
        // Copy all User Defined field functions

        Vector<UserFieldFunction> ffc;
        ffc = simArray[source].getFieldFunctionManager().getObjectsOf(UserFieldFunction.class);

        for (FieldFunction fF : ffc) {
            for (int i = source + 1; i < source + simArray.length; i++) {
                UserFieldFunction uFF0
                        = simArray[i%simArray.length].getFieldFunctionManager().createFieldFunction();
                uFF0.setPresentationName(fF.getPresentationName());
                uFF0.copyProperties(fF);
            }
        }
        
        // Copy all cartesian coordinate systems local to LAB.

        LabCoordinateSystem labCS0
                = simArray[source].getCoordinateSystemManager().getLabCoordinateSystem();

        Collection<CartesianCoordinateSystem> cscol = new ArrayList<CartesianCoordinateSystem>();

        cscol = labCS0.getLocalCoordinateSystemManager().getObjectsOf(CartesianCoordinateSystem.class);

        for (CoordinateSystem cs : cscol) {
            for (int i = source + 1; i < source + simArray.length; i++) {

                LabCoordinateSystem labCSi
                        = simArray[i%simArray.length].getCoordinateSystemManager().getLabCoordinateSystem();
                CartesianCoordinateSystem cartCS
                        = labCSi.getLocalCoordinateSystemManager().createLocalCoordinateSystem(CartesianCoordinateSystem.class);
                cartCS.setPresentationName(cs.getPresentationName());
                cartCS.copyProperties(cs);
            }
        }
        
        // Copy Physics continuum

        Vector<PhysicsContinuum> phcV;
        phcV = simArray[source].getContinuumManager().getObjectsOf(PhysicsContinuum.class);

        for (PhysicsContinuum physicsContinuum : phcV) {
            for (int i = source + 1; i < source + simArray.length; i++) {
                PhysicsContinuum phc0
                        = simArray[i%simArray.length].getContinuumManager().createContinuum(PhysicsContinuum.class);
                phc0.setPresentationName(physicsContinuum.getPresentationName());
                phc0.copyProperties(physicsContinuum);
            }
        }

    }
}
